{                                                                           }
{ File:       FAppearanceFrame.pas                                          }
{ Function:   Frame for Appearance tab in Options dialog in Greppola        }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003,2004 Rudy Velthuis                                   }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit FAppearanceFrame;

{$WARN UNIT_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Borland.Vcl.StdCtrls, Borland.Vcl.ExtCtrls,
  System.ComponentModel, MyExtCtrls;

type
  TAppearanceFrame = class(TFrame)
    TextFontBox: TShape;
    FileFontBox: TShape;
    FileFontCaption: TLabel;
    TextFontCaption: TLabel;
    TextFontLabel: TLabel;
    FileFontLabel: TLabel;
    FileFontButton: TButton;
    TextFontButton: TButton;
    BackgroundColorBox: TMyColorBox;
    BackgroundCaption: TLabel;
    FontDialog: TFontDialog;
    UnderlineColorBox: TMyColorBox;
    HighlightCaption: TLabel;
    UnderlineRadio: TRadioButton;
    BackgroundRadio: TRadioButton;
    HighlightStyleCaption: TLabel;
    procedure FileFontButtonClick(Sender: TObject);
    procedure TextFontButtonClick(Sender: TObject);
    procedure BackgroundColorBoxChange(Sender: TObject);
    procedure UnderlineColorBoxChange(Sender: TObject);
    procedure UnderlineRadioClick(Sender: TObject);
  private
    procedure SetFileFont(const Value: TFont);
    procedure SetTextFont(const Value: TFont);
    procedure SetFont(Font: TFont; ToFont: TFont; ALabel: TLabel);
    procedure ChangeFont(Font: TFont; ALabel: TLabel);
    procedure SetBackColor(const Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateOptions;
    procedure UpdateControls;
  end;

implementation

uses GreppolaOptions;

resourcestring
  StrFontNameAndSize = '%s, %d pt.';

{$R *.nfm}

procedure TAppearanceFrame.ChangeFont(Font: TFont; ALabel: TLabel);
begin
  FontDialog.Font.Assign(Font);
  if FontDialog.Execute then
    SetFont(FontDialog.Font, Font, ALabel);
end;

procedure TAppearanceFrame.TextFontButtonClick(Sender: TObject);
begin
  ChangeFont(GlobalOptions.Appearance.TextFont, TextFontLabel);
end;

procedure TAppearanceFrame.FileFontButtonClick(Sender: TObject);
begin
  ChangeFont(GlobalOptions.Appearance.FileFont, FileFontLabel);
end;

procedure TAppearanceFrame.SetFileFont(const Value: TFont);
begin
  SetFont(Value, GlobalOptions.Appearance.FileFont, FileFontLabel);
end;

procedure TAppearanceFrame.SetTextFont(const Value: TFont);
begin
  SetFont(Value, GlobalOptions.Appearance.TextFont, TextFontLabel);
end;

procedure TAppearanceFrame.SetFont(Font: TFont; ToFont: TFont; ALabel: TLabel);
begin
  ToFont.Assign(Font);
  ALabel.Caption := Format(StrFontNameAndSize, [Font.Name, Font.Size]);
  ALabel.Font := Font;
end;

procedure TAppearanceFrame.BackgroundColorBoxChange(Sender: TObject);
begin
  SetBackColor(BackgroundColorBox.Selected);
end;

// Workaround for a bug in TCustomColorBox.
// If cbPrettyNames is set, setting Selected does not (always?) change
// the value of the colour.
procedure SetPrettyColor(Box: TMyColorBox; Color: TColor);
begin
  Box.Style := Box.Style - [cbPrettyNames];
  try
    Box.Selected := Color;
  except // in case of custom color!
    Box.Items.Objects[0] := TObject(Color);
    Box.Selected := Color;
  end;
  Box.Style := Box.Style + [cbPrettyNames];
end;

procedure TAppearanceFrame.SetBackColor(const Value: TColor);
begin
  GlobalOptions.Appearance.BackColor := Value;
  FileFontBox.Brush.Color := Value;
  TextFontBox.Brush.Color := Value;
  SetPrettyColor(BackgroundColorBox, Value);
end;

procedure TAppearanceFrame.UnderlineColorBoxChange(Sender: TObject);
begin
  GlobalOptions.Appearance.UnderlineColor :=
    UnderlineColorBox.Selected;
end;

constructor TAppearanceFrame.Create(AOwner: TComponent);
begin
  inherited;
  UpdateControls;
end;

destructor TAppearanceFrame.Destroy;
begin
  inherited;
end;

procedure TAppearanceFrame.UnderlineRadioClick(Sender: TObject);
begin
  GlobalOptions.Appearance.HighlightStyle := BackgroundRadio.Checked;
end;

procedure TAppearanceFrame.UpdateOptions;
begin
  GlobalOptions.Appearance.HighlightStyle := BackgroundRadio.Checked;
end;

procedure TAppearanceFrame.UpdateControls;
begin
  FileFontBox.Brush.Color := GlobalOptions.Appearance.BackColor;
  TextFontBox.Brush.Color := GlobalOptions.Appearance.BackColor;
  SetPrettyColor(BackgroundColorBox, GlobalOptions.Appearance.BackColor);
  SetPrettyColor(UnderlineColorBox, GlobalOptions.Appearance.UnderlineColor);
  SetFileFont(GlobalOptions.Appearance.FileFont);
  BackgroundRadio.Checked := GlobalOptions.Appearance.HighlightStyle;
  SetTextFont(GlobalOptions.Appearance.TextFont);
end;

end.
