{                                                                           }
{ File:       FEditList.pas                                                 }
{ Function:   Form enabling the editing of the list of items in a TStrings. }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit FEditList;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Borland.Vcl.StdCtrls, System.ComponentModel,
  Borland.Vcl.CheckLst, Borland.Vcl.ExtCtrls;

type
  TEditListForm = class(TForm)
    ItemsListBox: TCheckListBox;
    DeleteButton: TButton;
    MoveUpButton: TButton;
    MoveDownButton: TButton;
    OkButton: TButton;
    CancelButton: TButton;
    HelpButton: TButton;
    ListCaptionLabel: TLabel;
    SortButton: TButton;
    AscendingRadio: TRadioButton;
    DescendingRadio: TRadioButton;
    ListLabel: TLabel;
    ClearButton: TButton;
    CheckedItemsLabel: TLabel;
    SelectedLabel: TLabel;
    LineShape: TShape;
    procedure SortButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure MoveUpButtonClick(Sender: TObject);
    procedure MoveDownButtonClick(Sender: TObject);
  private
    function CompareListItems(List: TStringList; Index1,
      Index2: Integer): Integer;
  public
    function GetItems: TStrings;
    function GetItemCount: Integer;
  public
    constructor Create(AOwner: TComponent; Items: TStrings;
      const ListCaption: string); reintroduce; overload;
    property Items: TStrings read GetItems;
    property ItemCount: Integer read GetItemCount;
  end;

function EditStrings(const ACaption: string; Items: TStrings): Boolean;

var
  EditListForm: TEditListForm;

implementation

{$R *.nfm}

function EditStrings(const ACaption: string; Items: TStrings): Boolean;
var
  EditListForm: TEditListForm;
begin
  EditListForm := TEditListForm.Create(nil, Items, ACaption);
  try
    Result := EditListForm.ShowModal = mrOK;
    if Result then
      Items.Assign(EditListForm.Items);
  finally
    EditListForm.Free;
  end;
end;

{ TEditListForm }

constructor TEditListForm.Create(AOwner: TComponent; Items: TStrings;
  const ListCaption: string);
begin
  inherited Create(AOwner);
  ListCaptionLabel.Caption := ListCaption;
  ItemsListBox.Items := Items;
end;

function TEditListForm.CompareListItems(List: TStringList;
  Index1, Index2: Integer): Integer;
begin
  Result := CompareText(List[Index1], List[Index2]);
  if DescendingRadio.Checked then
    Result := -Result;
end;

procedure TEditListForm.SortButtonClick(Sender: TObject);
var
  List: TStringList;
begin
  List := TStringList.Create;
  try
    List.Assign(ItemsListBox.Items);
    List.CustomSort(Self.CompareListItems);
    ItemsListBox.Items.Assign(List);
  finally
    List.Free;
  end;
end;

function TEditListForm.GetItems: TStrings;
begin
  Result := ItemsListBox.Items;
end;

function TEditListForm.GetItemCount: Integer;
begin
  Result := ItemsListBox.Count;
end;

procedure TEditListForm.DeleteButtonClick(Sender: TObject);
var
  I: Integer;
begin
  I := 0;
  while I < ItemsListBox.Count do
  begin
    if ItemsListBox.Checked[I] then
      ItemsListBox.Items.Delete(I)
    else
      Inc(I);
  end;
end;

procedure TEditListForm.ClearButtonClick(Sender: TObject);
begin
  ItemsListBox.Clear;
end;

procedure TEditListForm.MoveUpButtonClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := ItemsListBox.ItemIndex;
  if Index > 0 then
  begin
    ItemsListBox.Items.Move(Index, Index - 1);
    ItemsListBox.ItemIndex := Index - 1;
  end;
end;

procedure TEditListForm.MoveDownButtonClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := ItemsListBox.ItemIndex;
  if (Index >= 0) and (Index < ItemsListBox.Count - 1) then
  begin
    ItemsListBox.Items.Move(Index, Index + 1);
    ItemsListBox.ItemIndex := Index + 1;
  end;
end;

end.

