{                                                                           }
{ File:       FEditor.pas                                                   }
{ Function:   Simple editor for the Greppola application.                   }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit FEditor;

{$WARN UNIT_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdActns, ExtActns, ActnMenus, ActnList, ToolWin,
  ActnMan, ActnCtrls, XPStyleActnCtrls, ImgList, StdCtrls,
  StdStyleActnCtrls, System.ComponentModel, Borland.Vcl.Menus,
  FEditorFrame, FEditorSearch, Borland.Vcl.Themes,
  GreppolaUtils;

type
  TEditorForm = class(TForm)
    PageControl: TPageControl;
    FindDialog: TFindDialog;
    AllActions: TActionList;
    FileOpenAction: TAction;
    FileSaveAsAction: TAction;
    FilePrintSetupAction: TAction;
    FilePageSetupAction: TAction;
    FileCloseAction: TAction;
    EditCutAction: TAction;
    EditCopyAction: TAction;
    EditPasteAction: TAction;
    EditUndoAction: TAction;
    EditDeleteAction: TAction;
    SearchSearchAction: TAction;
    EditorMainMenu: TMainMenu;
    FileMenu: TMenuItem;
    EditMenu: TMenuItem;
    SearchMenu: TMenuItem;
    OpenItem: TMenuItem;
    SaveAsItem: TMenuItem;
    PrintSetupItem: TMenuItem;
    PageSetupItem: TMenuItem;
    ExitItem: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    UndoItem: TMenuItem;
    DeleteItem: TMenuItem;
    FileBreak1: TMenuItem;
    EditBreak: TMenuItem;
    OpenDialog: TOpenDialog;
    Images: TImageList;
    SaveDialog: TSaveDialog;
    EditToolBar: TToolBar;
    HotImages: TImageList;
    DisabledImages: TImageList;
    OpenToolButton: TToolButton;
    SaveAsToolButton: TToolButton;
    CutToolButton: TToolButton;
    CopyToolButton: TToolButton;
    FileBreak2: TMenuItem;
    FirstSeparatorToolButton: TToolButton;
    PasteToolButton: TToolButton;
    SearchItem: TMenuItem;
    SecondSeparatorToolButton: TToolButton;
    SearchToolButton: TToolButton;
    ThirdSeparatorToolButton: TToolButton;
    CloseToolButton: TToolButton;
    StatusBar: TStatusBar;
    DeleteToolButton: TToolButton;
    SearchAgainAction: TAction;
    SearchAgainItem: TMenuItem;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    FontDialog: TFontDialog;
    PrinterSetupDialog: TPrinterSetupDialog;
    PrintDialog: TPrintDialog;
    SystemFindDialog: TFindDialog;
    PageSetupDialog: TPageSetupDialog;
    FilePrintAction: TAction;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FileOpenActionExecute(Sender: TObject);
    procedure FileCloseActionExecute(Sender: TObject);
    procedure FileSaveAsActionExecute(Sender: TObject);
    procedure EditCutActionExecute(Sender: TObject);
    procedure EditCopyActionExecute(Sender: TObject);
    procedure EditPasteActionExecute(Sender: TObject);
    procedure EditUndoActionExecute(Sender: TObject);
    procedure EditDeleteActionExecute(Sender: TObject);
    procedure FilePrintActionExecute(Sender: TObject);
    procedure FilePageSetupActionExecute(Sender: TObject);
    procedure SearchSearchActionExecute(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure SearchAgainActionExecute(Sender: TObject);
    procedure FilePrintSetupActionExecute(Sender: TObject);
  private
    FSearchForm: TEditorSearchForm;
    FRegex: STRRegex;
    FOptions: TSearchOptions;
    procedure FrameChange(Sender: TObject; Line, Column: Integer; Inserting, HasSelection, CanUndo: Boolean);
  public
    procedure AddFile(FileName: string; MatchData: TMatchData);
    procedure CheckSelection(HasSelection, CanUndo: Boolean);
    function GetCurrentFrame: TEditorFrame;
  end;

var
  EditorForm: TEditorForm;

implementation

{$R *.nfm}

uses
  GreppolaConsts, Clipbrd;

procedure TEditorForm.AddFile(FileName: string; MatchData: TMatchData);
var
  Sheet: TTabSheet;
  Frame: TEditorFrame;
  I: Integer;
begin
  for I := 0 to PageControl.PageCount - 1 do
  begin
    Sheet := PageControl.Pages[I];
    Frame := Sheet.Controls[0] as TEditorFrame;
    if Frame.FileName = FileName then
    begin
      Windows.SetFocus(Sheet.Handle);
      PageControl.ActivePage := Sheet;
      Frame.Regex := MatchData.Regex;
      Frame.SelectLine(MatchData.Line, MatchData.Start, MatchData.Length);
      CheckSelection(Frame.HasSelection, Frame.CanUndo);
      Exit;
    end;
  end;

  // Only reached if no file with given FileName found.
  Sheet := TTabSheet.Create(Self);
  Sheet.Caption := ExtractFileName(FileName);
  Sheet.PageControl := PageControl;
  Frame := TEditorFrame.Create(Self);
  Frame.Parent := Sheet;
  Frame.Name := '';
  Frame.Align := alClient;
  Frame.Regex := MatchData.Regex;
  Frame.OnChange := FrameChange;
  Frame.OpenFile(FileName);
  PageControl.ActivePage := Sheet;
  Windows.SetFocus(Sheet.Handle);
  Frame.SelectLine(MatchData.Line, MatchData.Start, MatchData.Length);
  CheckSelection(Frame.HasSelection, Frame.CanUndo);
end;

procedure TEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caHide;
end;

procedure TEditorForm.FileOpenActionExecute(Sender: TObject);
begin
  if OpenDialog.Execute then
    AddFile(OpenDialog.FileName, TMatchData.Create(-1, 0, 0, nil));
end;

procedure TEditorForm.FileCloseActionExecute(Sender: TObject);
begin
  if Assigned(PageControl.ActivePage) then
    PageControl.ActivePage.Free;
  if PageControl.PageCount < 1 then
    Close;
end;

procedure TEditorForm.FileSaveAsActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  if Assigned(Frame) then
  begin
    SaveDialog.FileName := Frame.FileName;
    if SaveDialog.Execute then
    begin
      Frame.FileName := SaveDialog.FileName;
      PageControl.ActivePage.Caption := ExtractFileName(SaveDialog.FileName);
    end;
  end;
end;

procedure TEditorForm.EditCutActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  ClipBoard.AsText := Frame.RichEdit.SelText;
  Frame.RichEdit.SelText := '';
end;

procedure TEditorForm.EditCopyActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  ClipBoard.AsText := Frame.RichEdit.SelText;
end;

procedure TEditorForm.EditPasteActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  if ClipBoard.HasFormat(CF_TEXT) then
    Frame.RichEdit.SelText := Clipboard.AsText;
end;

procedure TEditorForm.EditUndoActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  Frame.RichEdit.Undo;
end;

procedure TEditorForm.EditDeleteActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  Frame.RichEdit.SelText := '';
end;

procedure TEditorForm.FilePrintActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  if Assigned(Frame) then
    Frame.RichEdit.Print(Frame.FileName);
end;

// Yes, this could be a little more sophisticated.
procedure TEditorForm.FilePrintSetupActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  if Assigned(Frame) then
    if PrintDialog.Execute(Handle) then
      Frame.RichEdit.Print(Frame.FileName);
end;

procedure TEditorForm.FilePageSetupActionExecute(Sender: TObject);
begin
  //
end;

procedure TEditorForm.SearchSearchActionExecute(Sender: TObject);
var
  Options: STRRegexOptions;
  Frame: TEditorFrame;
begin
  if not Assigned(FSearchForm) then
    FSearchForm := TEditorSearchForm.Create(Self);
  if FSearchForm.ShowModal = mrOK then
  begin
    FOptions := FSearchForm.Options;
    if FOptions.CaseSensitive then
      Options := STRRegexOptions.None
    else
      Options := STRRegexOptions.IgnoreCase;
    Frame := GetCurrentFrame;
    if Assigned(Frame) then
    begin
      FRegex := STRRegex.Create(FOptions.SearchString, Options);
      Frame.Search(FRegex, FOptions.WholeWords);
    end;
  end;
end;

procedure TEditorForm.SearchAgainActionExecute(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  if Assigned(Frame) then
    if Assigned(Frame.Regex) then
    begin
      Frame.Search(Frame.Regex, FOptions.WholeWords);
      FRegex := Frame.Regex;
      Frame.Regex := nil;
    end
    else
      Frame.Search(FRegex, FOptions.WholeWords);
end;

procedure TEditorForm.CheckSelection(HasSelection, CanUndo: Boolean);
begin
  EditCutAction.Enabled := HasSelection;
  EditCopyAction.Enabled := HasSelection;
  EditPasteAction.Enabled := ClipBoard.HasFormat(CF_TEXT);
  EditUndoAction.Enabled := CanUndo;
end;

function TEditorForm.GetCurrentFrame: TEditorFrame;
var
  I: Integer;
begin
  Result := nil;
  if Assigned(PageControl.ActivePage) then
    for I := 0 to PageControl.ActivePage.ControlCount - 1 do
      if PageControl.ActivePage.Controls[I] is TEditorFrame then
      begin
        Result := TEditorFrame(PageControl.ActivePage.Controls[I]);
        Exit;
      end;
end;

procedure TEditorForm.PageControlChange(Sender: TObject);
var
  Frame: TEditorFrame;
begin
  Frame := GetCurrentFrame;
  FocusControl(Frame.RichEdit);
  CheckSelection(Frame.HasSelection, Frame.CanUndo);
end;

procedure TEditorForm.FrameChange(Sender: TObject; Line, Column: Integer;
  Inserting: Boolean; HasSelection, CanUndo: Boolean);
begin
  CheckSelection(HasSelection, CanUndo);
  StatusBar.Panels[0].Text := Format('%d:%d', [Line, Column]);
  if Inserting then
    StatusBar.Panels[1].Text := GreppolaConsts.RsrcInsert
  else
    StatusBar.Panels[1].Text := GreppolaConsts.RsrcOverflow;
end;

end.
