{                                                                           }
{ File:       FMain.pas                                                     }
{ Function:   Main form for Greppola                                        }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003,2004 Rudy Velthuis                                   }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit FMain;

interface

{$WARN UNIT_PLATFORM OFF}      

uses
  SysUtils, Graphics, Controls, Forms, StdCtrls, ComCtrls, ExtCtrls,
  Classes, FileCtrl, ImgList, SearchThread, FEditor, FOptions,
  System.ComponentModel, MRUCombos, System.Text.RegularExpressions, XPMan,
  GreppolaUtils, Borland.Vcl.Buttons, Borland.Vcl.Menus;

type

  TSearchOption = (soCaseSensitive, soWholeWords, soSubdirectories, soTextFiles,
                   soSystemFolders);
  TSearchOptions = set of TSearchOption;

  TMainGrepForm = class(TForm)
    ResultsTreeView: TTreeView;
    BrowseButton: TButton;
    FileMaskLabel: TLabel;
    ExpressionLabel: TLabel;
    DirectoryLabel: TLabel;
    SubdirsCheck: TCheckBox;
    CaseCheck: TCheckBox;
    WordsCheck: TCheckBox;
    LineNumsCheck: TCheckBox;
    SearchingLabel: TLabel;
    SearchCaption: TLabel;
    OptionsCaption: TLabel;
    SearchButton: TButton;
    ExitButton: TButton;
    HelpButton: TButton;
    ResultsCaption: TLabel;
    OpenButton: TButton;
    ShellImages: TImageList;
    ViewButton: TButton;
    LineNumExtraLabel: TLabel;
    ExpressionCombo: TMRUComboBox;
    FileMaskCombo: TMRUComboBox;
    DirectoryCombo: TMRUComboBox;
    TopLineShape: TShape;
    BottomLineShape: TShape;
    RightLineShape: TShape;
    OptionsButton: TButton;
    TextFilesCheck: TCheckBox;
    EditSearchListButton: TSpeedButton;
    EditMaskListButton: TSpeedButton;
    EditDirectoryListButton: TSpeedButton;
    TreeViewPopupMenu: TPopupMenu;
    ExpandAllItem: TMenuItem;
    CollapseAllItem: TMenuItem;
    FirstSeparatorItem: TMenuItem;
    ViewSelectedItem: TMenuItem;
    OpenSelectedItem: TMenuItem;
    SystemFoldersCheck: TCheckBox;
    DateCautionLabel: TLabel;
    SizeCautionLabel: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Bevel4: TBevel;
    procedure SystemFoldersCheckClick(Sender: TObject);
    procedure CaseCheckClick(Sender: TObject);
    procedure SearchButtonClick(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure BrowseButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure ResultsTreeViewCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure ResultsTreeViewAdvancedCustomDrawItem(
      Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
      Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
    procedure ViewButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EditSearchListButtonClick(Sender: TObject);
    procedure EditMaskListButtonClick(Sender: TObject);
    procedure EditDirectoryListButtonClick(Sender: TObject);
    procedure OptionsButtonClick(Sender: TObject);
    procedure ExpandAllItemClick(Sender: TObject);
    procedure CollapseAllItemClick(Sender: TObject);
    procedure ViewSelectedItemClick(Sender: TObject);
    procedure ResultsTreeViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    FSearchThread: TSearchThread;
    FIsSearching: Boolean;
    FNode: TTreeNode;
    FIcons: TList;
    FOptions: TSearchOptions;
    procedure UpdateOptions;
    procedure CheckOption(Option: TSearchOption; Value: Boolean);
    procedure SetSearching(const Value: string);
    procedure ThreadTerminate(Sender: TObject);
    procedure LoadPresets;
    procedure SavePresets;
    procedure EditList(const ACaption: string; Combo: TCustomComboBox);
    procedure View(Node: TTreeNode);
    procedure UpdateCautionLabels;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddDirectory(Directory: string; List: TStrings);
    procedure AddFile(const FileName: string);
    procedure AddLine(const Line: string; LineNum, Start, Length: Integer;
      Regex: STRRegex);
    property Searching: string write SetSearching;
  end;

var
  MainGrepForm: TMainGrepForm;

implementation

uses
  GreppolaConsts, Windows, ShellAPI, Dialogs, CommCtrl,
  IniFiles, FEditList, System.Windows.Forms, FAppearanceFrame,
  GreppolaOptions;

resourcestring
  StrSearchExpressions = 'Search Expressions';
  StrFileMasks = 'File Masks';
  StrDirectories = 'Directories';
  StrTextFont = 'Courier New';
  StrFileFont = 'Tahoma';
  StrDate = 'Date limits';
  StrSize = 'Size limits';

{$R *.nfm}

procedure TMainGrepForm.SearchButtonClick(Sender: TObject);
begin
  if FIsSearching then
    FSearchThread.Terminate
  else
  begin
    if (DirectoryCombo.Text = '') or (ExpressionCombo.Text = '') then
      Exit;

    FIsSearching := True;
    SearchButton.Caption := RsrcCaptionStop;
    try
      ResultsTreeView.Items.Clear;
      ShellImages.Clear;
      FIcons.Clear;

      FSearchThread := TSearchThread.Create(DirectoryCombo.Text,
        FileMaskCombo.Text, SubDirsCheck.Checked, ExpressionCombo.Text,
        CaseCheck.Checked, WordsCheck.Checked, TextFilesCheck.Checked);
      FSearchThread.OnTerminate := ThreadTerminate;
      FSearchThread.OnAddDirectory := AddDirectory;
      FSearchThread.FreeOnTerminate := True;
      FSearchThread.Resume;
    finally
      SearchButton.Enabled := True;
    end;
  end;
end;                                           

procedure TMainGrepForm.ExitButtonClick(Sender: TObject);
begin
  if Assigned(FSearchThread) then
  begin
    FSearchThread.Terminate;
    FSearchThread.WaitFor;
  end;
  Close;
end;

procedure TMainGrepForm.BrowseButtonClick(Sender: TObject);
var
  Browser: System.Windows.Forms.FolderBrowserDialog;
begin
  Browser := System.Windows.Forms.FolderBrowserDialog.Create;
  if DirectoryCombo.ItemIndex >= 0 then
    Browser.SelectedPath := DirectoryCombo.Items[DirectoryCombo.ItemIndex];
  if Browser.ShowDialog = System.Windows.Forms.DialogResult.OK then
  begin
    DirectoryCombo.Items.Add(Browser.SelectedPath);
    DirectoryCombo.ItemIndex := 0;
  end;
end;

procedure TMainGrepForm.SetSearching(const Value: string);
begin
  SearchingLabel.Caption := Value;
end;

procedure TMainGrepForm.FormCreate(Sender: TObject);
begin
  FIsSearching := False;
  LoadPresets;
  UpdateCautionLabels;
end;

procedure TMainGrepForm.HelpButtonClick(Sender: TObject);
begin
  ShowMessage(SNotImplemented);
end;

procedure TMainGrepForm.ThreadTerminate(Sender: TObject);
begin
  FIsSearching := False;
  SearchButton.Caption := RsrcCaptionSearch;
  with FSearchThread do
    Searching := Format(SSearchedAndFound,
      [SearchedCount, LineCount, FoundCount]);
  FSearchThread := nil;
end;

procedure TMainGrepForm.OpenButtonClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  Node := ResultsTreeView.Selected;
  if Assigned(Node) then
  begin
    if Assigned(Node.Parent) then
      Node := Node.Parent;
    ShellExecute(0, 'open', Node.Text, '', '', SW_SHOW); // do not localize
  end;
end;

procedure TMainGrepForm.ResultsTreeViewCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  var DefaultDraw: Boolean);

  procedure SetFont(Dest, Src: TFont);
  begin
    Dest.Name := Src.Name;
    Dest.Size := Src.Size;
    if not (cdsSelected in State) then
      Dest.Color := Src.Color;
  end;

begin
  // If Data is assigned, then it must be a text subnode
  if Assigned(Node.Data) then
    SetFont(Sender.Canvas.Font, GlobalOptions.Appearance.TextFont)
  else
    SetFont(Sender.Canvas.Font, GlobalOptions.Appearance.FileFont);
end;

procedure TMainGrepForm.ResultsTreeViewAdvancedCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
var
  Rect: TRect;
  Start, Stop: Integer;
  ExtraCanvas: TControlCanvas;
  HighlightRect: TRect;
begin
  if (Stage = cdPostPaint) and Assigned(Node.Data) then
  begin
    ExtraCanvas := TControlCanvas.Create;
    try
      Rect := Node.DisplayRect(True);
      ExtraCanvas.Control := Sender;
      ExtraCanvas.Font.Assign(GlobalOptions.Appearance.TextFont);

      Start := TMatchData(Node.Data).Start;
      if LineNumsCheck.Checked then
        Inc(Start, 8);
      Stop := Start + TMatchData(Node.Data).Length;

      if GlobalOptions.Appearance.HighlightStyle then
      begin
        ExtraCanvas.Brush.Color := GlobalOptions.Appearance.UnderlineColor;
        ExtraCanvas.Brush.Style := bsSolid;
        HighlightRect := TRect.Create(
          Rect.Left + 2 + ExtraCanvas.TextWidth(Copy(Node.Text, 1, Start)),
          Rect.Top,
          Rect.Left + 2 + ExtraCanvas.TextWidth(Copy(Node.Text, 1, Stop)),
          Rect.Bottom);
        ExtraCanvas.FillRect(HighlightRect);
        ExtraCanvas.TextOut(
          Rect.Left + 2 + ExtraCanvas.TextWidth(Copy(Node.Text, 1, Start)),
          Rect.Top + 1,
          Copy(Node.Text, Start + 1, Stop - Start));
      end
      else
      begin
        ExtraCanvas.Pen.Color := GlobalOptions.Appearance.UnderlineColor;
        ExtraCanvas.MoveTo(Rect.Left + 2 +
          ExtraCanvas.TextWidth(Copy(Node.Text, 1, Start)), Rect.Bottom - 1);
        ExtraCanvas.LineTo(Rect.Left + 2 +
          ExtraCanvas.TextWidth(Copy(Node.Text, 1, Stop)), Rect.Bottom - 1);
      end;
    finally
      ExtraCanvas.Free;
    end;
  end;
end;

procedure TMainGrepForm.View(Node: TTreeNode);
var
  MatchData: TMatchData;
begin
  if not Assigned(Node) then Exit;
  if not Assigned(EditorForm) then
    EditorForm := TEditorForm.Create(Self);
  if Assigned(Node.Data) then
  begin
    MatchData := Node.Data as TMatchData;
    Node := Node.Parent;
  end
  else
    MatchData := TMatchData.Create(-1, 0, 0, nil);
  EditorForm.AddFile(Node.Text, MatchData);
  EditorForm.Show;
end;

procedure TMainGrepForm.ViewButtonClick(Sender: TObject);
begin
  View(ResultsTreeView.Selected);
end;

procedure TMainGrepForm.AddFile(const FileName: string);
var
  Info: TSHFileInfo;
  Index: Integer;
begin
  FNode := ResultsTreeView.Items.Add(nil, FileName);

  // Get icon info and store index in list
  SHGetFileInfo(FileName, 0, Info, SizeOf(Info),
    SHGFI_SYSICONINDEX or SHGFI_ICON or SHGFI_SMALLICON);
  Index := FIcons.IndexOf(TObject(Info.iIcon));
  if Index = -1 then
  begin
    ImageList_AddIcon(ShellImages.Handle, Info.hIcon);
    Index := FIcons.Add(TObject(Info.iIcon));
  end;
  FNode.ImageIndex := Index;
  FNode.SelectedIndex := Index;
end;

procedure TMainGrepForm.AddLine(const Line: string; LineNum, Start,
  Length: Integer; Regex: STRRegex);
var
  Node: TTreeNode;
  Match: TMatchData;
begin
  if LineNumsCheck.Checked then
    Node := ResultsTreeView.Items.AddChild(FNode, Format('%6d: %s',
      [LineNum + 1, Line]))
  else
    Node := ResultsTreeView.Items.AddChild(FNode, Line);
  Node.ImageIndex := -1;
  Node.SelectedIndex := -1;
  Match := TMatchData.Create(LineNum, Start, Length, Regex);
  Node.Data := Match;
end;

procedure TMainGrepForm.UpdateCautionLabels;
begin
  DateCautionLabel.Visible := GlobalOptions.Search.ChangedOption <> scoUnknown;
  SizeCautionLabel.Visible := GlobalOptions.Search.SizeOption <> ssoUnknown;
end;

constructor TMainGrepForm.Create(AOwner: TComponent);
begin
  inherited;
  FIcons := TList.Create;
  with GlobalOptions.Appearance do
  begin
    if not Assigned(TextFont) then
    begin
      TextFont := TFont.Create;
      TextFont.Name := StrTextFont;
      TextFont.Size := 9;
      TextFont.Color := clBlue;
    end;
    if not Assigned(FileFont) then
    begin
      FileFont := TFont.Create;
      FileFont.Name := StrFileFont;
      FileFont.Size := 8;
    end;
  end;
end;

destructor TMainGrepForm.Destroy;
begin
  FIcons.Free;
  inherited;
end;

procedure TMainGrepForm.AddDirectory(Directory: string; List: TStrings);
var
  I: Integer;
  Match: TMatchData;
begin
  Searching := Format(SSearching, [Directory]);
  if List.Count = 0 then
    Exit;
  for I := 0 to List.Count - 1 do
  begin
    Match := TMatchData(List.Objects[I]);
    if Assigned(Match) then
      AddLine(List[I], Match.Line, Match.Start, Match.Length, Match.Regex)
    else
      AddFile(List[I]);
  end;
end;

function GetIniFileName: string;
begin
  Result := ExtractFilePath(Forms.Application.ExeName) +
            CIniFileName;
end;

procedure LoadCombo(Combo: TMRUComboBox; AFile: TIniFile; Section, BaseIdent: string);
begin
  Combo.LoadFromIniFile(AFile, Section, BaseIdent);
  Combo.ItemIndex := 0;
  Combo.SelLength := 0;
end;

procedure TMainGrepForm.LoadPresets;
var
  Ini: TIniFile;
begin
  if not FileExists(GetIniFileName) then
    SavePresets;
  Ini := TIniFile.Create(GetIniFileName);
  try
    SetBounds(Ini.ReadInteger(CPosition, CLeft, Left),
              Ini.ReadInteger(CPosition, CTop, Top),
              Ini.ReadInteger(CPosition, CWidth, Width),
              Ini.ReadInteger(CPosition, CHeight, Height));
    WindowState := TWindowState(Ini.ReadInteger(CPosition, CWindowState,
      Byte(wsNormal)));
    LoadCombo(ExpressionCombo, Ini, CSearchExpressions, CItem);
    LoadCombo(FileMaskCombo, Ini, CFileMasks, CItem);
    LoadCombo(DirectoryCombo, Ini, CDirectories, CItem);
    SubdirsCheck.Checked := Ini.ReadBool(COptions, CSubdirectories, True);
    WordsCheck.Checked := Ini.ReadBool(COptions, CWholeWords, False);
    CaseCheck.Checked := Ini.ReadBool(COptions, CCaseSensitive, False);
    TextFilesCheck.Checked := Ini.ReadBool(COptions, CTextFiles, True);
    SystemFoldersCheck.Checked := Ini.ReadBool(COptions, CSystemFolders, True);
    UpdateOptions;
    LineNumsCheck.Checked := Ini.ReadBool(COptions, CLineNumbers, True);
    GlobalOptions.Load(Ini);
    ResultsTreeView.Color := GlobalOptions.Appearance.BackColor;
  finally
    Ini.Free;
  end;
end;

procedure TMainGrepForm.SavePresets;
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(GetIniFileName);
  try
    ExpressionCombo.SaveToIniFile(Ini, CSearchExpressions, CItem);
    FileMaskCombo.SaveToIniFile(Ini, CFileMasks, CItem);
    DirectoryCombo.SaveToIniFile(Ini, CDirectories, CItem);
    Ini.WriteBool(COptions, CSubdirectories, SubdirsCheck.Checked);
    Ini.WriteBool(COptions, CWholeWords, WordsCheck.Checked);
    Ini.WriteBool(COptions, CCaseSensitive, CaseCheck.Checked);
    Ini.WriteBool(COptions, CTextFiles, TextFilesCheck.Checked);
    Ini.WriteBool(COptions, CSystemFolders, SystemFoldersCheck.Checked);
    Ini.WriteBool(COptions, CLineNumbers, LineNumsCheck.Checked);
    Ini.WriteInteger(CPosition, CWindowState, Byte(WindowState));
    GlobalOptions.Save(Ini);
    if WindowState = wsNormal then
    begin
      Ini.WriteInteger(CPosition, CLeft, Left);
      Ini.WriteInteger(CPosition, CTop, Top);
      Ini.WriteInteger(CPosition, CWidth, Width);
      Ini.WriteInteger(CPosition, CHeight, Height);
    end;
  finally
    Ini.Free;
  end;
end;

procedure TMainGrepForm.FormDestroy(Sender: TObject);
begin
  SavePresets;
end;

procedure TMainGrepForm.EditList(const ACaption: string;
  Combo: TCustomComboBox);
begin
  if EditStrings(ACaption, Combo.Items) then
    Combo.ItemIndex := 0;
end;

procedure TMainGrepForm.EditSearchListButtonClick(Sender: TObject);
begin
  EditList(StrSearchExpressions, ExpressionCombo);
end;

procedure TMainGrepForm.EditMaskListButtonClick(Sender: TObject);
begin
  EditList(StrFileMasks, FileMaskCombo);
end;

procedure TMainGrepForm.EditDirectoryListButtonClick(Sender: TObject);
begin
  EditList(StrDirectories, DirectoryCombo);
end;

procedure TMainGrepForm.OptionsButtonClick(Sender: TObject);
var
  OptionsForm: TOptionsForm;
begin
  OptionsForm := TOptionsForm.Create(Self);
  try
    if OptionsForm.ShowModal = mrOK then
    begin
      ResultsTreeView.Color := GlobalOptions.Appearance.BackColor;
      ResultsTreeView.Invalidate;
    end;
  finally
    OptionsForm.Free;
  end;
  UpdateCautionLabels;
end;

procedure TMainGrepForm.ExpandAllItemClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  Node := ResultsTreeView.Items.GetFirstNode;
  while Assigned(Node) do
  begin
    Node.Expand(True);
    Node := Node.GetNextSibling;
  end;
end;

procedure TMainGrepForm.CollapseAllItemClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  Node := ResultsTreeView.Items.GetFirstNode;
  while Assigned(Node) do
  begin
    Node.Collapse(True);
    Node := Node.GetNextSibling;
  end;
end;

procedure TMainGrepForm.ViewSelectedItemClick(Sender: TObject);
begin
  View(ResultsTreeView.Selected);
end;

procedure TMainGrepForm.ResultsTreeViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
  Scr, Clnt: TPoint;
begin
  if Button = mbRight then
  begin
    GetCursorPos(Scr);
    Clnt := ResultsTreeView.ScreenToClient(Scr);
    Node := ResultsTreeView.GetNodeAt(Clnt.X, Clnt.Y);
    if Assigned(Node) then
      ResultsTreeView.Selected := Node;
  end;
end;

procedure TMainGrepForm.UpdateOptions;
begin
  CheckOption(soCaseSensitive, CaseCheck.Checked);
  CheckOption(soWholeWords, WordsCheck.Checked);
  CheckOption(soSubdirectories, SubdirsCheck.Checked);
  CheckOption(soTextFiles, TextFilesCheck.Checked);
  CheckOption(soSystemFolders, SystemFoldersCheck.Checked);
end;

procedure TMainGrepForm.CheckOption(Option: TSearchOption; Value: Boolean);
begin
  if Value then
    Include(FOptions, Option)
  else
    Exclude(FOptions, Option);
end;

procedure TMainGrepForm.CaseCheckClick(Sender: TObject);
begin
  CheckOption(soCaseSensitive, CaseCheck.Checked);
end;

procedure TMainGrepForm.SystemFoldersCheckClick(Sender: TObject);
begin
  CheckOption(soSystemFolders, SystemFoldersCheck.Checked);
end;

end.
