unit FSearchOptionsFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Borland.Vcl.ComCtrls, Borland.Vcl.StdCtrls, System.ComponentModel,
  Borland.Vcl.ExtCtrls, GreppolaOptions, Borland.Vcl.Spin;

{
  When was the file changed:
  (*) unknown
  ( ) In the last week
  ( ) In the last month
  ( ) In the last year
  ( ) Date
  [Edit date|v] // or creation date or access date
  From: [         ]
  To:   [         ]

  What is the size of the file:
  (*) unknown
  ( ) small (smaller than 100 KB)
  ( ) middle (smaller than 1 MB)
  ( ) large (larger than 1 MB)
  ( ) size in KB:
  [at leastv] [      |+-] // at least or at most
}

type
  TSearchOptionsFrame = class(TFrame)
    Panel1: TPanel;
    Label2: TLabel;
    SizeUnknownRadio: TRadioButton;
    SizeSmallRadio: TRadioButton;
    SizeMiddleRadio: TRadioButton;
    SizeLargeRadio: TRadioButton;
    SizeKBRadio: TRadioButton;
    SizeTypeCombo: TComboBox;
    ChangedPanel: TPanel;
    ChangedUserDateRadio: TRadioButton;
    ChangedLastMonthRadio: TRadioButton;
    ChangedLastWeekRadio: TRadioButton;
    ChangedUnknownRadio: TRadioButton;
    Label1: TLabel;
    ChangedFromLabel: TLabel;
    ChangedToLabel: TLabel;
    FromDateTimePicker: TDateTimePicker;
    ToDateTimePicker: TDateTimePicker;
    DateTypeCombo: TComboBox;
    Shape1: TShape;
    ChangedLastYearRadio: TRadioButton;
    SizeSizeSpinEdit: TSpinEdit;
    procedure SizeSizeSpinEditChange(Sender: TObject);
    procedure SizeTypeComboChange(Sender: TObject);
    procedure ChangedLastYearRadioClick(Sender: TObject);
    procedure SizeKBRadioClick(Sender: TObject);
    procedure SizeLargeRadioClick(Sender: TObject);
    procedure SizeMiddleRadioClick(Sender: TObject);
    procedure SizeSmallRadioClick(Sender: TObject);
    procedure SizeUnknownRadioClick(Sender: TObject);
    procedure DateTypeComboChange(Sender: TObject);
    procedure ToDateTimePickerChange(Sender: TObject);
    procedure FromDateTimePickerChange(Sender: TObject);
    procedure ChangedPanelDblClick(Sender: TObject);
    procedure ChangedPanelClick(Sender: TObject);
    procedure ChangedUserDateRadioClick(Sender: TObject);
    procedure ChangedLastMonthRadioClick(Sender: TObject);
    procedure ChangedLastWeekRadioClick(Sender: TObject);
    procedure ChangedUnknownRadioClick(Sender: TObject);
  private
    FSearch: TSearchOptions;
    FUpdating: Boolean;
    procedure SetChangedOption(Option: TSearchChangedOption);
    procedure SetSizeOption(Option: TSearchSizeOption);
  public
    procedure UpdateControls;
    procedure UpdateOptions;
    constructor Create(AOwner: TComponent); override;
  end;

implementation

{$R *.nfm}

constructor TSearchOptionsFrame.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TSearchOptionsFrame.SetChangedOption(Option: TSearchChangedOption);
var
  Enable: Boolean;
begin
  if FUpdating then
    Exit;
  FSearch.ChangedOption := Option;
  Enable := Option = scoUserDate;
  DateTypeCombo.Enabled := Enable;
  FromDateTimePicker.Enabled := Enable;
  ToDateTimePicker.Enabled := Enable;
  ChangedFromLabel.Enabled := Enable;
  ChangedToLabel.Enabled := Enable;
end;

procedure TSearchOptionsFrame.SetSizeOption(Option: TSearchSizeOption);
begin
  FSearch.SizeOption := Option;
  SizeTypeCombo.Enabled := Option = ssoSize;
  SizeSizeSpinEdit.Enabled := Option = ssoSize;
end;

procedure TSearchOptionsFrame.ChangedUnknownRadioClick(Sender: TObject);
begin
  SetChangedOption(scoUnknown);
end;

procedure TSearchOptionsFrame.ChangedLastWeekRadioClick(Sender: TObject);
begin
  SetChangedOption(scoLastWeek);
end;

procedure TSearchOptionsFrame.ChangedLastMonthRadioClick(Sender: TObject);
begin
  SetChangedOption(scoLastMonth);
end;

procedure TSearchOptionsFrame.ChangedLastYearRadioClick(Sender: TObject);
begin
  SetChangedOption(scoLastYear);
end;

procedure TSearchOptionsFrame.ChangedUserDateRadioClick(Sender: TObject);
begin
  SetChangedOption(scoUserDate);
end;

procedure TSearchOptionsFrame.ChangedPanelClick(Sender: TObject);
begin
  SetChangedOption(scoUnknown);
end;

procedure TSearchOptionsFrame.ChangedPanelDblClick(Sender: TObject);
begin
  SetChangedOption(scoUnknown);
end;

procedure TSearchOptionsFrame.FromDateTimePickerChange(Sender: TObject);
begin
  FSearch.FromDate := FromDateTimePicker.Date;
end;

procedure TSearchOptionsFrame.ToDateTimePickerChange(Sender: TObject);
begin
  FSearch.ToDate := ToDateTimePicker.Date;
end;

procedure TSearchOptionsFrame.DateTypeComboChange(Sender: TObject);
begin
  FSearch.DateOption := TSearchDateOption(DateTypeCombo.ItemIndex);
end;

procedure TSearchOptionsFrame.UpdateControls;
begin
  if FUpdating then
    Exit;
  FUpdating := True;
  try
    FSearch := GlobalOptions.Search;
    case FSearch.ChangedOption of
      scoUnknown:
        ChangedUnknownRadio.Checked := True;
      scoLastWeek:
        ChangedLastWeekRadio.Checked := True;
      scoLastMonth:
        ChangedLastMonthRadio.Checked := True;
      scoLastYear:
        ChangedLastYearRadio.Checked := True;
      scoUserDate:
        begin
          ChangedUserDateRadio.Checked := True;
          DateTypeCombo.Enabled := True;
          FromDateTimePicker.Enabled := True;
          ToDateTimePicker.Enabled := True;
        end;
    end;
    case FSearch.SizeOption of
      ssoUnknown:
        SizeUnknownRadio.Checked := True;
      ssoSmall:
        SizeSmallRadio.Checked := True;
      ssoMiddle:
        SizeMiddleRadio.Checked := True;
      ssoLarge:
        SizeLargeRadio.Checked := True;
      ssoSize:
        begin
          SizeKBRadio.Checked := True;
          SizeTypeCombo.Enabled := True;
          SizeSizeSpinEdit.Enabled := True;
        end;
    end;
    DateTypeCombo.ItemIndex := Integer(FSearch.DateOption);
    FromDateTimePicker.Date := FSearch.FromDate;
    ToDateTimePicker.Date := FSearch.ToDate;
    SizeTypeCombo.ItemIndex := Integer(FSearch.SizeSizeOption);
    SizeSizeSpinEdit.Value := FSearch.Size;
  finally
    FUpdating := False;
  end;
end;

procedure TSearchOptionsFrame.SizeUnknownRadioClick(Sender: TObject);
begin
  SetSizeOption(ssoUnknown);
end;

procedure TSearchOptionsFrame.SizeSmallRadioClick(Sender: TObject);
begin
  SetSizeOption(ssoSmall);
end;

procedure TSearchOptionsFrame.SizeMiddleRadioClick(Sender: TObject);
begin
  SetSizeOption(ssoMiddle);
end;

procedure TSearchOptionsFrame.SizeLargeRadioClick(Sender: TObject);
begin
  SetSizeOption(ssoLarge);
end;

procedure TSearchOptionsFrame.SizeKBRadioClick(Sender: TObject);
begin
  SetSizeOption(ssoSize);
end;

procedure TSearchOptionsFrame.SizeTypeComboChange(Sender: TObject);
begin
  FSearch.SizeSizeOption := TSearchSizeSizeOption(SizeTypeCombo.ItemIndex);
end;

procedure TSearchOptionsFrame.SizeSizeSpinEditChange(Sender: TObject);
begin
  FSearch.Size := SizeSizeSpinEdit.Value;
end;

procedure TSearchOptionsFrame.UpdateOptions;
begin
  GlobalOptions.Search := FSearch;
end;

end.

