{                                                                           }
{ File:       GreppolaUtils.pas                                             }
{ Function:   Utilities for the Greppola grep search tool.                  }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit GreppolaUtils;

interface

{$IFNDEF CLR}
{$MESSAGE ERROR 'Sorry, only .NET'}
{$ENDIF}

uses
  System.Text.RegularExpressions;

type
  // Aliases
  STRRegex = System.Text.RegularExpressions.Regex;
  STRMatch = System.Text.RegularExpressions.Match;
  STRRegexOptions = System.Text.RegularExpressions.RegexOptions;

  // Data stored with a string with a regular expression match.
  TMatchData = class(TObject)
  public
    Regex: STRRegex;
    Line: Integer;
    Start: Integer;
    Length: Integer;
    constructor Create(ALine, AStart, ALength: Integer; ARegex: STRRegex);
  end;

  // Search options
  TSearchOptions = record
    SearchString: string;
    CaseSensitive: Boolean;
    WholeWords: Boolean;
    Multiline: Boolean;
    RegularExpressions: Boolean;
  end;

type
  CharSet = set of AnsiChar;

var
  // Can't use set of Char in .NET, since that is a set of AnsiChar,
  // while string contains WideChars.
  WordChars: string = '_0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
                      'abcdefghijklmnopqrstuvwxyz';

// Checks first 4096 bytes of a file to check for bytes 0..8 or 14..31. If one
// of these is found, Result is false, otherwise, Result is True.
function FileIsText(FileName: string): Boolean;

// Checks to see if the match is a single word, which means that all characters
// in the match are in WordChars, and the characters surrounding it (if any)
// are NOT in WordChars.
function WordCheck(const Line: string; const Match: TMatchData): Boolean;

procedure ShowMessage(Text: string);

implementation

uses
  SysUtils, Classes, Dialogs;

procedure ShowMessage(Text: string);
begin
  Dialogs.ShowMessage(Text);
end;

function FileIsText(FileName: string): Boolean;
type
  TByteArray = array of Byte;
const
  CBufSize = 4096;
var
  F: TFileStream;
  Buffer: TByteArray;
  BytesRead, I: Integer;
begin
  Buffer := New(TByteArray, CBufSize);
  F := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
  try
    Result := False;
    BytesRead := F.Read(Buffer, CBufSize);
    for I := 0 to BytesRead - 1 do
      if (Buffer[I] in [0..8, 14..31]) then
        Exit;
    Result := True;
  finally
    F.Free;
  end;
end;

function WordCheck(const Line: string; const Match: TMatchData): Boolean;
var
  I: Integer;
begin
  Result := True;
  // Instead of Pos, this uses String.IndexOf(Char)
  if ((Match.Start > 0) and (WordChars.IndexOf(Line[Match.Start]) >= 0)) or
     ((Match.Start + Match.Length < Length(Line)) and (WordChars.IndexOf(Line[Match.Start + Match.Length + 1]) >= 0)) then
    Result := False
  else
    for I := Match.Start + 1 to Match.Start + Match.Length do
      if WordChars.IndexOf(Line[I]) < 0 then
      begin
        Result := False;
        Break;
      end;
end;

{ TMatchData }

constructor TMatchData.Create(ALine, AStart, ALength: Integer;
  ARegex: STRRegex);
begin
  inherited Create;
  Line := ALine;
  Start := AStart;
  Length := ALength;
  Regex := ARegex;
end;

end.

